// For Hatmoose ENJOY
// Big_P Big_P

// Controller Pinout
// Looking at the male socket front on 
// .------------------------------------------.
//  \  [U]    [D]   [L]    [R]      [+5V]    /
//   \   [B/A]  [SEL] [GND]   [C/START]     /
//    '------------------------------------'



// Arduino Pins
// A6-A7 are not available 

const int PIN_SELECT = 13;

const int PIN_IN_UP = A0;
const int PIN_IN_DOWN = A1;
const int PIN_IN_LEFT = A2;
const int PIN_IN_RIGHT = A3;
const int PIN_IN_BTN_B_A = A4;
const int PIN_IN_BTN_C_START = A5;

const int PIN_OUT_UP = 2;
const int PIN_OUT_DOWN = 3;
const int PIN_OUT_LEFT = 4;
const int PIN_OUT_RIGHT = 5;
const int PIN_OUT_BTN_1 = 6;
const int PIN_OUT_BTN_2 = 7;
const int PIN_OUT_BTN_3 = 8;
const int PIN_OUT_BTN_4 = 9;
const int PIN_OUT_BTN_5 = 10;
const int PIN_OUT_BTN_6 = 11;
const int PIN_OUT_BTN_START = 12;

// controller select line state
bool select;

// button hold counter
int starthold;
const int START_HOLD_TICKS = 500;

void setup() {
  pinMode(PIN_SELECT, OUTPUT);

  // pullups used to prevent erratic inputs when the controller is unplugged
  pinMode(PIN_IN_UP, INPUT_PULLUP);
  pinMode(PIN_IN_DOWN, INPUT_PULLUP);
  pinMode(PIN_IN_LEFT, INPUT_PULLUP);
  pinMode(PIN_IN_RIGHT, INPUT_PULLUP);
  pinMode(PIN_IN_BTN_C_START, INPUT_PULLUP);
  pinMode(PIN_IN_BTN_B_A, INPUT_PULLUP);

  // until a button is pressed this keeps the pins in a high impedance state. 
  pinMode(PIN_OUT_UP, INPUT_PULLUP);
  pinMode(PIN_OUT_DOWN, INPUT_PULLUP);
  pinMode(PIN_OUT_LEFT, INPUT_PULLUP);
  pinMode(PIN_OUT_RIGHT, INPUT_PULLUP);
  pinMode(PIN_OUT_BTN_1, INPUT_PULLUP);
  pinMode(PIN_OUT_BTN_2, INPUT_PULLUP);
  pinMode(PIN_OUT_BTN_3, INPUT_PULLUP);
  pinMode(PIN_OUT_BTN_4, INPUT_PULLUP);
  pinMode(PIN_OUT_BTN_5, INPUT_PULLUP);
  pinMode(PIN_OUT_BTN_6, INPUT_PULLUP);

  // start can be tied to a secondary button so use output only when a
  // controller button pressed to prevent shorting the output to ground
  pinMode(PIN_OUT_BTN_START, INPUT_PULLUP);
  
  select = 0;
  Serial.begin(230400); 
}

void loop() {
  // flip SELECT pin
  select = !select;

  // let things settle
  digitalWrite(PIN_SELECT, select);
  delay(2);
  
  /*
  Pin  Connection with select low  Connection with select high
  ---  --------------------------  ---------------------------
   1   joypad up signal            joypad up signal
   2   joypad down signal          joypad down signal
   3   logic low (ground)          joypad left signal
   4   logic low (ground)          joypad right signal
   6   Button A signal             Button B signal
   9   Start button signal         Button C signal
  */

  // regardless of SELECT value, U and D is the same in 3 buttom mode

  // Up
  if (!digitalRead(PIN_IN_UP)) pinMode(PIN_OUT_UP, OUTPUT);
  else pinMode(PIN_OUT_UP, INPUT_PULLUP);
  digitalWrite(PIN_OUT_UP, digitalRead(PIN_IN_UP));
  
  // Down
  if (!digitalRead(PIN_IN_DOWN)) pinMode(PIN_OUT_DOWN, OUTPUT);
  else pinMode(PIN_OUT_DOWN, INPUT_PULLUP);
  digitalWrite(PIN_OUT_DOWN, digitalRead(PIN_IN_DOWN));
  
  // if SELECT is high
  if (select){

    // Button 2
    if (!digitalRead(PIN_IN_BTN_B_A)) pinMode(PIN_OUT_BTN_2, OUTPUT);
    else pinMode(PIN_OUT_BTN_2, INPUT_PULLUP);
    digitalWrite(PIN_OUT_BTN_2, digitalRead(PIN_IN_BTN_B_A));        
    
    // Button 3
    if (!digitalRead(PIN_IN_BTN_C_START)) pinMode(PIN_OUT_BTN_3, OUTPUT);
    else pinMode(PIN_OUT_BTN_3, INPUT_PULLUP);   
    digitalWrite(PIN_OUT_BTN_3, digitalRead(PIN_IN_BTN_C_START));

    // Left
    if (!digitalRead(PIN_IN_LEFT)) pinMode(PIN_OUT_LEFT, OUTPUT);
    else pinMode(PIN_OUT_LEFT, INPUT_PULLUP);
    digitalWrite(PIN_OUT_LEFT, digitalRead(PIN_IN_LEFT));
       
    // Right
    if (!digitalRead(PIN_IN_RIGHT)) pinMode(PIN_OUT_RIGHT, OUTPUT);
    else pinMode(PIN_OUT_RIGHT, INPUT_PULLUP);
    digitalWrite(PIN_OUT_RIGHT, digitalRead(PIN_IN_RIGHT));
  }
    
  // if SELECT is low
  else {

    // Button 1
    if (!digitalRead(PIN_IN_BTN_B_A)) pinMode(PIN_OUT_BTN_1, OUTPUT);
    else pinMode(PIN_OUT_BTN_1, INPUT_PULLUP);
    digitalWrite(PIN_OUT_BTN_1, digitalRead(PIN_IN_BTN_B_A));
    
    // Button 4
    if (!digitalRead(PIN_IN_BTN_C_START)) pinMode(PIN_OUT_BTN_4, OUTPUT);
    else pinMode(PIN_OUT_BTN_4, INPUT_PULLUP);
    digitalWrite(PIN_OUT_BTN_4, digitalRead(PIN_IN_BTN_C_START));
  }

  // Start Button
  // alternate start function (holding down start for some ticks
  if (!digitalRead(PIN_OUT_BTN_4)){
    starthold++;  
  }
  else starthold = 0;

  // turn on start by momentarily setting the button to output mode
  if (starthold > START_HOLD_TICKS){
    pinMode(PIN_OUT_BTN_START, OUTPUT);
    digitalWrite(PIN_OUT_BTN_START, 0);
  }
  else{
    pinMode(PIN_OUT_BTN_START, INPUT_PULLUP);
    digitalWrite(PIN_OUT_BTN_START, 1);
  }

  // print state via serial (can be disabled)
  print_state();
}

void print_state(){
  //Serial.print("*-UDLRBC-UDLR123456S\n"); 

  select ? Serial.print("H") : Serial.print("L");

  Serial.print("-");

  digitalRead(PIN_IN_UP) ? Serial.print("U") : Serial.print(" ");
  digitalRead(PIN_IN_DOWN) ? Serial.print("D") : Serial.print(" ");
  digitalRead(PIN_IN_LEFT) ? Serial.print("L") : Serial.print(" ");
  digitalRead(PIN_IN_RIGHT) ? Serial.print("R") : Serial.print(" ");

  digitalRead(PIN_IN_BTN_B_A) ? Serial.print("B") : Serial.print(" ");
  digitalRead(PIN_IN_BTN_C_START) ? Serial.print("C") : Serial.print(" ");
  
  Serial.print("-");

  digitalRead(PIN_OUT_UP) ? Serial.print("U") : Serial.print(" ");
  digitalRead(PIN_OUT_DOWN) ? Serial.print("D") : Serial.print(" ");
  digitalRead(PIN_OUT_LEFT) ? Serial.print("L") : Serial.print(" ");
  digitalRead(PIN_OUT_RIGHT) ? Serial.print("R") : Serial.print(" ");

  digitalRead(PIN_OUT_BTN_1) ? Serial.print("1") : Serial.print(" ");
  digitalRead(PIN_OUT_BTN_2) ? Serial.print("2") : Serial.print(" ");
  digitalRead(PIN_OUT_BTN_3) ? Serial.print("3") : Serial.print(" ");
  digitalRead(PIN_OUT_BTN_4) ? Serial.print("4") : Serial.print(" ");
  digitalRead(PIN_OUT_BTN_5) ? Serial.print("5") : Serial.print(" ");
  digitalRead(PIN_OUT_BTN_6) ? Serial.print("6") : Serial.print(" ");
  digitalRead(PIN_OUT_BTN_START) ? Serial.print("S") : Serial.print(" ");

  Serial.print("-");
  Serial.print(starthold, DEC);
  Serial.print("\n"); 
}

