// For Hatmoose Enjoy
// Big_P Big_P

/ Controller Pinout
// Looking at the male socket front on 
// .------------------------------------------.
//  \  [U]    [D]   [L]    [R]      [+5V]    /
//   \   [B/A]  [SEL] [GND]   [C/START]     /
//    '------------------------------------'


// Arduino Pins
// A6-A7 are not usable 

const int PIN_SELECT = 13;

const int PIN_IN_UP = A0;
const int PIN_IN_DOWN = A1;
const int PIN_IN_LEFT = A2;
const int PIN_IN_RIGHT = A3;
const int PIN_IN_BTN_B_A = A4;
const int PIN_IN_BTN_C_START = A5;

const int PIN_OUT_UP = 2;
const int PIN_OUT_DOWN = 3;
const int PIN_OUT_LEFT = 4;
const int PIN_OUT_RIGHT = 5;

// Pinout set so top 3 buttons are punch and bottom 3 kick in Street Fighter 2
// This may not work for every game but I think it is a good place to start can
// use button remap from this starting point if needed.
const int PIN_OUT_BTN_1 = 9;
const int PIN_OUT_BTN_2 = 10;
const int PIN_OUT_BTN_3 = 11;
const int PIN_OUT_BTN_4 = 6;
const int PIN_OUT_BTN_5 = 7;
const int PIN_OUT_BTN_6 = 8;
const int PIN_OUT_BTN_START = 12;

// controller select line state
int select;

void setup() {
  pinMode(PIN_SELECT, OUTPUT);
  
  pinMode(PIN_IN_UP, INPUT_PULLUP);
  pinMode(PIN_IN_DOWN, INPUT_PULLUP);
  pinMode(PIN_IN_LEFT, INPUT_PULLUP);
  pinMode(PIN_IN_RIGHT, INPUT_PULLUP);
  pinMode(PIN_IN_BTN_C_START, INPUT_PULLUP);
  pinMode(PIN_IN_BTN_B_A, INPUT_PULLUP);

  // until a button is pressed this keeps the pins in a high impedance state. 
  pinMode(PIN_OUT_UP, INPUT);
  pinMode(PIN_OUT_DOWN, INPUT);
  pinMode(PIN_OUT_LEFT, INPUT);
  pinMode(PIN_OUT_RIGHT, INPUT);
  pinMode(PIN_OUT_BTN_1, INPUT);
  pinMode(PIN_OUT_BTN_2, INPUT);
  pinMode(PIN_OUT_BTN_3, INPUT);
  pinMode(PIN_OUT_BTN_4, INPUT);
  pinMode(PIN_OUT_BTN_5, INPUT);
  pinMode(PIN_OUT_BTN_6, INPUT);
  pinMode(PIN_OUT_BTN_START, INPUT);

  select = 0;
  Serial.begin(230400); 
}

void loop() {
  // start with a select pin high state and wait a bit to let things settle
  select = 1;
  digitalWrite(PIN_SELECT, select);
  delay(10);

  // This bit was a PITA to work out
  /*
  Pin  Select: low  Select: high  Select: pulse-3
  --- ------------  ------------  ---------------
    1  joypad up     joypad up     button Z
    2  joypad down   joypad down   button Y
    3  logic low     joypad left   button X
    4  logic low     joypad right
    6  button A      button B
    9  start button  button C
  */

  // read UP and DOWN values in select high state 

  // Up
  if (!digitalRead(PIN_IN_UP)){
    digitalWrite(PIN_OUT_UP, 0);
    pinMode(PIN_OUT_UP, OUTPUT);
  }
  else pinMode(PIN_OUT_UP, INPUT);
  
  
  // Down
  if (!digitalRead(PIN_IN_DOWN)){
    digitalWrite(PIN_OUT_DOWN, 0);
    pinMode(PIN_OUT_DOWN, OUTPUT);
  }
  else pinMode(PIN_OUT_DOWN, INPUT);
  
   
  // SELECT is still high
  // Button 2
  if (!digitalRead(PIN_IN_BTN_B_A)){
    digitalWrite(PIN_OUT_BTN_2, 0);       
    pinMode(PIN_OUT_BTN_2, OUTPUT);
  }
  else pinMode(PIN_OUT_BTN_2, INPUT);
   
  
  // Button 3
  if (!digitalRead(PIN_IN_BTN_C_START)){
    digitalWrite(PIN_OUT_BTN_3, 0);
    pinMode(PIN_OUT_BTN_3, OUTPUT);
  }
  else pinMode(PIN_OUT_BTN_3, INPUT);
  
  // Left
  if (!digitalRead(PIN_IN_LEFT)){
    digitalWrite(PIN_OUT_LEFT, 0);
    pinMode(PIN_OUT_LEFT, OUTPUT);
  }
  else pinMode(PIN_OUT_LEFT, INPUT);
     
  // Right
  if (!digitalRead(PIN_IN_RIGHT)){
    digitalWrite(PIN_OUT_RIGHT, 0);
    pinMode(PIN_OUT_RIGHT, OUTPUT);
  }
  else pinMode(PIN_OUT_RIGHT, INPUT);
  
  
  print_state();

  // set select pin state to low and wait a bit to let things settle
  select = 0;
  digitalWrite(PIN_SELECT, select);
  delay(10);

  // now SELECT is low

  // Button 1
  if (!digitalRead(PIN_IN_BTN_B_A)){
    digitalWrite(PIN_OUT_BTN_1, 0);
    pinMode(PIN_OUT_BTN_1, OUTPUT);
  }
  else pinMode(PIN_OUT_BTN_1, INPUT);
   
  // Start Button
  if (!digitalRead(PIN_IN_BTN_C_START)){
    digitalWrite(PIN_OUT_BTN_START, 0);
    pinMode(PIN_OUT_BTN_START, OUTPUT);
  }
  else pinMode(PIN_OUT_BTN_START, INPUT);
  

  print_state();

  // now trigger the 3-pulse mode of the controller
  select = 3;
  digitalWrite(PIN_SELECT, 1);
  delayMicroseconds(15);
  digitalWrite(PIN_SELECT, 0);
  delayMicroseconds(15);
  digitalWrite(PIN_SELECT, 1);
  delayMicroseconds(15);
  digitalWrite(PIN_SELECT, 0);
  delayMicroseconds(15);
  digitalWrite(PIN_SELECT, 1);
  delayMicroseconds(15);
  //digitalWrite(PIN_SELECT, 0);
  //delayMicroseconds(15);

  // Button 4
  if (!digitalRead(PIN_IN_LEFT)){
    digitalWrite(PIN_OUT_BTN_4, 0);
    pinMode(PIN_OUT_BTN_4, OUTPUT);
  }
  else pinMode(PIN_OUT_BTN_4, INPUT);
  

  // Button 5
  if (!digitalRead(PIN_IN_DOWN)){
    digitalWrite(PIN_OUT_BTN_5, 0);
    pinMode(PIN_OUT_BTN_5, OUTPUT);
  }
  else pinMode(PIN_OUT_BTN_5, INPUT);
      
  // Button 6
  if (!digitalRead(PIN_IN_UP)){
    digitalWrite(PIN_OUT_BTN_6, 0);
    pinMode(PIN_OUT_BTN_6, OUTPUT);
  }
  else pinMode(PIN_OUT_BTN_6, INPUT);
  
  // print state via serial (can be disabled)  
  print_state();
}

void print_state(){
  //Serial.print("*-UDLRBC-UDLR123456S\n"); 

  if (select == 0) Serial.print("L");
  else if (select == 1) Serial.print("H"); 
  else Serial.print("3");

  Serial.print("-");

  digitalRead(PIN_IN_UP) ? Serial.print("U") : Serial.print(" ");
  digitalRead(PIN_IN_DOWN) ? Serial.print("D") : Serial.print(" ");
  digitalRead(PIN_IN_LEFT) ? Serial.print("L") : Serial.print(" ");
  digitalRead(PIN_IN_RIGHT) ? Serial.print("R") : Serial.print(" ");

  digitalRead(PIN_IN_BTN_B_A) ? Serial.print("B") : Serial.print(" ");
  digitalRead(PIN_IN_BTN_C_START) ? Serial.print("C") : Serial.print(" ");
  
  Serial.print("-");

  digitalRead(PIN_OUT_UP) ? Serial.print("U") : Serial.print(" ");
  digitalRead(PIN_OUT_DOWN) ? Serial.print("D") : Serial.print(" ");
  digitalRead(PIN_OUT_LEFT) ? Serial.print("L") : Serial.print(" ");
  digitalRead(PIN_OUT_RIGHT) ? Serial.print("R") : Serial.print(" ");

  digitalRead(PIN_OUT_BTN_1) ? Serial.print("1") : Serial.print(" ");
  digitalRead(PIN_OUT_BTN_2) ? Serial.print("2") : Serial.print(" ");
  digitalRead(PIN_OUT_BTN_3) ? Serial.print("3") : Serial.print(" ");
  digitalRead(PIN_OUT_BTN_4) ? Serial.print("4") : Serial.print(" ");
  digitalRead(PIN_OUT_BTN_5) ? Serial.print("5") : Serial.print(" ");
  digitalRead(PIN_OUT_BTN_6) ? Serial.print("6") : Serial.print(" ");
  digitalRead(PIN_OUT_BTN_START) ? Serial.print("S") : Serial.print(" ");

  Serial.print("\n"); 
}

